@@ -1994,7 +1994,7 @@ class CMacrosResolverGeneral {
 	 *
 	 * @return array
 	 */
-	protected static function getSupportedHostInventoryMacrosMap(): array {
+	public static function getSupportedHostInventoryMacrosMap(): array {
 		return [
 			'{INVENTORY.ALIAS}' => 'alias',
 			'{INVENTORY.ASSET.TAG}' => 'asset_tag',
@@ -2707,7 +2707,8 @@ class CMacrosResolverGeneral {
 							}
 							elseif ($data['context'] !== null && count($global_macros[$data['macro']]['regex'])) {
 								foreach ($global_macros[$data['macro']]['regex'] as $regex => $val) {
-									if (preg_match('/'.self::handleSlashEscaping($regex).'/', $data['context'])) {
+									if (preg_match(
+											'/'.CRegexHelper::handleSlashEscaping($regex).'/', $data['context'])) {
 										$data['value']['value'] = $val;
 										break;
 									}
@@ -2794,7 +2795,7 @@ class CMacrosResolverGeneral {
 				// Searching context coincidence, if regex array not empty.
 				elseif ($context !== null && count($host_macros[$hostid][$macro]['regex'])) {
 					foreach ($host_macros[$hostid][$macro]['regex'] as $regex => $val) {
-						if (preg_match('/'.self::handleSlashEscaping($regex).'/', $context) === 1) {
+						if (preg_match('/'.CRegexHelper::handleSlashEscaping($regex).'/', $context) === 1) {
 							return [
 								'value' => $val,
 								'value_default' => $value_default
@@ -2918,34 +2919,6 @@ class CMacrosResolverGeneral {
 		return $macro_values;
 	}
 
-	/**
-	 * Escape slashes in the regular expression based on preceding backslashes.
-	 *
-	 * @param string $regex
-	 *
-	 * @return string
-	 */
-	private static function handleSlashEscaping(string $regex): string {
-		$formatted_regex = '';
-		$backslash_count = 0;
-
-		for ($p = 0; isset($regex[$p]); $p++) {
-			if ($regex[$p] === '\\') {
-				$backslash_count++;
-			}
-			else {
-				if ($regex[$p] === '/' && $backslash_count % 2 == 0) {
-					$formatted_regex .= '\\';
-				}
-				$backslash_count = 0;
-			}
-
-			$formatted_regex .= $regex[$p];
-		}
-
-		return $formatted_regex;
-	}
-
 	/**
 	 * Get macro value refer by type.
 	 *
