@@ -1,6 +1,6 @@
 <?php
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -125,7 +125,8 @@ function make_event_details(array $event, array $allowed) {
 				if ($allowed['ui_correlation']) {
 					$correlation_name = (new CLink($correlations[0]['name'],
 						(new CUrl('zabbix.php'))
-							->setArgument('action', 'correlation.edit')
+							->setArgument('action', 'popup')
+							->setArgument('popup', 'correlation.edit')
 							->setArgument('correlationid', $correlations[0]['correlationid'])
 							->getUrl()
 					))->addClass(ZBX_STYLE_LINK_ALT);
@@ -170,7 +171,7 @@ function make_event_details(array $event, array $allowed) {
 	$tags = makeTags([$event]);
 
 	$table
-		->addRow([_('Tags'), $tags[$event['eventid']]])
+		->addRow([_('Tags'), (new CDiv($tags[$event['eventid']]))->addClass(ZBX_STYLE_TAGS_WRAPPER)])
 		->addRow([_('Description'), (new CDiv(zbx_str2links($event['comments'])))->addClass(ZBX_STYLE_WORDBREAK)]);
 
 	if ($event['cause_eventid'] == 0) {
@@ -435,13 +436,16 @@ function make_small_eventlist(array $startEvent, array $allowed) {
 		 */
 		addTriggerValueStyle($cell_status, $value, $value_clock, $is_acknowledged);
 
+		$problem_update_url = (new CUrl('zabbix.php'))
+			->setArgument('action', 'popup')
+			->setArgument('popup', 'acknowledge.edit')
+			->setArgument('eventids[]', $event['eventid'])
+			->getUrl();
+
 		// Create acknowledge link.
 		$problem_update_link = ($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
 				|| $can_be_closed || $allowed['suppress_problems'] || $allowed['rank_change'])
-			? (new CLink(_('Update')))
-				->addClass(ZBX_STYLE_LINK_ALT)
-				->setAttribute('data-eventid', $event['eventid'])
-				->onClick('acknowledgePopUp({eventids: [this.dataset.eventid]}, this);')
+			? (new CLink(_('Update'), $problem_update_url))->addClass(ZBX_STYLE_LINK_ALT)
 			: new CSpan(_('Update'));
 
 		$table->addRow([
@@ -710,6 +714,7 @@ function makeTags(array $list, bool $html = true, string $key = 'eventid', int $
 								']);'
 							)
 							->addClass(ZBX_STYLE_BTN_TAG)
+							->addClass(ZBX_STYLE_TAG)
 							->setHint(getTagString($tag), '', false);
 					}
 					else {
@@ -746,6 +751,7 @@ function makeTags(array $list, bool $html = true, string $key = 'eventid', int $
 								']);'
 							)
 							->addClass(ZBX_STYLE_BTN_TAG)
+							->addClass(ZBX_STYLE_TAG)
 							->setHint(getTagString($tag), '', false);
 					}
 					else {
@@ -756,7 +762,7 @@ function makeTags(array $list, bool $html = true, string $key = 'eventid', int $
 				}
 
 				$tags[$element[$key]][] = (new CButtonIcon(ZBX_ICON_MORE))
-					->setHint($hint_content, ZBX_STYLE_HINTBOX_WRAP);
+					->setHint($hint_content, ZBX_STYLE_HINTBOX_WRAP . ' ' . ZBX_STYLE_TAGS_WRAPPER);
 			}
 		}
 		else {
