@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -43,10 +43,15 @@ jQuery(function($) {
 	endpoint.setArgument('action', 'timeselector.update');
 	endpoint.setArgument('type', PAGE_TYPE_TEXT_RETURN_JSON);
 
-	$.subscribe('timeselector.rangechange timeselector.decrement timeselector.increment timeselector.zoomout' +
-		' timeselector.rangeoffset',
-		timeSelectorEventHandler
-	);
+	$.subscribe(`
+		timeselector.rangechange
+		timeselector.decrement
+		timeselector.increment
+		timeselector.zoomout
+		timeselector.rangeoffset
+	`, timeSelectorEventHandler);
+
+	$.subscribe('timeselector.update-ui', checkDisableTimeSelectorUI);
 
 	element.from.keydown(submitChangeHandler);
 	element.to.keydown(submitChangeHandler);
@@ -581,7 +586,7 @@ jQuery(function($) {
 			});
 	}
 
-	if (!$container.data('disable-initial-check')) {
+	if (!$container.data('manual-setup')) {
 		checkDisableTimeSelectorUI();
 	}
 });
@@ -794,15 +799,6 @@ var timeControl = {
 		this.processObjects();
 	},
 
-	disableAllSBox: function() {
-		jQuery.each(this.objectList, function(i, obj) {
-			if (obj.loadSBox == 1) {
-				jQuery('#'+obj.containerid).removeClass('dashboard-widget-graph-link');
-			}
-		});
-		jQuery(document).off('dblclick mousedown', 'img');
-	},
-
 	/**
 	 * Update object timeline data. Will reload page when timeConrol.refreshPage is set to true.
 	 *
