@@ -1,28 +1,25 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
-jQuery(function($) {
+(function($) {
 	var ZBX_STYLE_CLASS = 'multiselect-control';
 	const MS_ACTION_POPUP = 0;
 	const MS_ACTION_AUTOSUGGEST = 1;
 
+	const FILTER_PRESELECT_ACCEPT_ID = 'id';
+
 	/**
 	 * Multi select helper.
 	 *
@@ -34,8 +31,8 @@ jQuery(function($) {
 	$.fn.multiSelectHelper = function(options) {
 		options = $.extend({objectOptions: {}}, options);
 
-		var curl = new Curl('jsrpc.php', false);
-		curl.setArgument('type', 11); // PAGE_TYPE_TEXT_RETURN_JSON
+		var curl = new Curl('jsrpc.php');
+		curl.setArgument('type', PAGE_TYPE_TEXT_RETURN_JSON);
 		curl.setArgument('method', 'multiselect.get');
 		curl.setArgument('object_name', options.object_name);
 
@@ -46,7 +43,9 @@ jQuery(function($) {
 		options.url = curl.getUrl();
 
 		return this.each(function() {
-			$(this).empty().multiSelect(options);
+			$(this).empty();
+			this.dataset.params = JSON.stringify(options);
+			$(this).multiSelect();
 		});
 	};
 
@@ -70,7 +69,8 @@ jQuery(function($) {
 				data.push({
 					id: id,
 					name: item.name,
-					prefix: (typeof item.prefix === 'undefined') ? '' : item.prefix
+					prefix: item.prefix === undefined ? '' : item.prefix,
+					isNew: item.isNew !== undefined && item.isNew
 				});
 			}
 
@@ -90,8 +90,8 @@ jQuery(function($) {
 		/**
 		 * Insert outside data
 		 *
-		 * @param {array} items           Multiselect value object.
-		 * @param {bool}  trigger_change  (optional) Either to trigger element on-change event once data added. True by default.
+		 * @param {array}   items           Multiselect value object.
+		 * @param {boolean} trigger_change  (optional) Either to trigger element on-change event once data added. True by default.
 		 *
 		 * @return jQuery
 		 */
@@ -110,6 +110,8 @@ jQuery(function($) {
 					}
 				}
 
+				$obj.trigger('before-add', ms);
+
 				for (var i = 0, l = items.length; i < l; i++) {
 					addSelected($obj, items[i]);
 				}
@@ -118,6 +120,18 @@ jQuery(function($) {
 			});
 		},
 
+		getSearch: function() {
+			let search = '';
+
+			this.each(function() {
+				const ms = $(this).data('multiSelect');
+
+				search = ms.values.search.toLowerCase().replace(/[*]+/g, '')
+			});
+
+			return search;
+		},
+
 		/**
 		 * Enable multi select UI control.
 		 *
@@ -128,13 +142,15 @@ jQuery(function($) {
 				var $obj = $(this),
 					ms = $obj.data('multiSelect');
 
-				if (ms.options.disabled === true) {
-					$obj.removeAttr('aria-disabled');
-					$('.multiselect-list', $obj).removeClass('disabled');
-					$('.multiselect-button > button', $obj.parent()).prop('disabled', false);
-					$obj.append(makeMultiSelectInput($obj));
+				$obj.removeAttr('aria-disabled');
+				$('.multiselect-list', $obj).removeClass('disabled');
+				$('.multiselect-button', $obj.parent()).prop('disabled', false);
+				$('.multiselect-optional-select-button', $obj.parent()).prop('disabled', false);
+				$('input', $obj).prop('disabled', false);
 
+				if (ms.options.disabled === true) {
 					ms.options.disabled = false;
+					$obj.append(makeMultiSelectInput($obj));
 
 					cleanSearch($obj);
 				}
@@ -151,19 +167,41 @@ jQuery(function($) {
 				var $obj = $(this),
 					ms = $obj.data('multiSelect');
 
-				if (ms.options.disabled === false) {
-					$obj.attr('aria-disabled', true);
-					$('.multiselect-list', $obj).addClass('disabled');
-					$('.multiselect-button > button', $obj.parent()).prop('disabled', true);
-					$('input[type="text"]', $obj).remove();
+				$obj.attr('aria-disabled', true);
+				$('.multiselect-list', $obj).addClass('disabled');
+				$('.multiselect-button', $obj.parent()).prop('disabled', true);
+				$('.multiselect-optional-select-button', $obj.parent()).prop('disabled', true);
+				$('input', $obj).prop('disabled', true);
 
+				if (ms.options.disabled === false) {
 					ms.options.disabled = true;
+					$('input[type="text"]', $obj).remove();
 
 					cleanSearch($obj);
 				}
 			});
 		},
 
+		/**
+		 * Remove select object value.
+		 *
+		 * @param {string} id
+		 *
+		 * @return jQuery
+		 */
+		removeSelected: function(id) {
+			return this.each(function() {
+				var $obj = $(this),
+					ms = $obj.data('multiSelect');
+
+				removeSelected($obj, id);
+
+				cleanSearch($obj);
+
+				$obj.trigger('change', ms);
+			});
+		},
+
 		/**
 		 * Clean multi select object values.
 		 *
@@ -202,6 +240,8 @@ jQuery(function($) {
 					}
 				}
 
+				cleanSearch($obj);
+
 				if (addNew_modified) {
 					/*
 					 * When modifying the "addNew" option, few things must be done:
@@ -210,15 +250,16 @@ jQuery(function($) {
 					 *      Note: hidden and disabled items will not submit to the server.
 					 *   3. The "change" trigger must fire.
 					 */
+					const new_item_pattern = ms.options.newItemName
+						? `input[name="${ms.options.newItemName}"]`
+						: 'input[name*="[new]"]';
 
-					cleanSearch($obj);
-
-					$('input[name*="[new]"]', $obj)
+					$(new_item_pattern, $obj)
 						.prop('disabled', !ms.options['addNew'])
 						.each(function() {
 							var id = $(this).val();
 							$('.selected li[data-id]', $obj).each(function() {
-								if ($(this).data('id') == id) {
+								if (this.dataset.id === id) {
 									$(this).toggle(ms.options['addNew']);
 								}
 							});
@@ -245,72 +286,174 @@ jQuery(function($) {
 			});
 
 			return ret;
+		},
+
+		/**
+		 * @return HTMLElement
+		 */
+		getSelectButton: function() {
+			var ret = null;
+
+			this.each(function() {
+				var $obj = $(this);
+
+				if ($obj.data('multiSelect') !== undefined) {
+					if ($obj.data('multiSelect').select_button !== null) {
+						ret = $obj.data('multiSelect').select_button[0];
+					}
+
+					return false;
+				}
+			});
+
+			return ret;
+		},
+
+		/**
+		 * @param array entries  IDs to mark disabled.
+		 */
+		setDisabledEntries: function(entries) {
+			this.each(function() {
+				const $obj = $(this);
+				const ms = $obj.data('multiSelect');
+
+				if (ms?.options.popup.parameters !== undefined) {
+					ms.options.popup.parameters.disableids = entries;
+				}
+			});
+		},
+
+		addOptionalSelect: function(label, callback) {
+			this.each(function() {
+				const $obj = $(this);
+
+				if ($obj.data('multiSelect') !== undefined) {
+					addOptionalSelect($obj, label, callback);
+
+					return false;
+				}
+			});
+		},
+
+		openSelectPopup: function(event_target) {
+			this.each(function() {
+				const $obj = $(this);
+				const ms = $obj.data('multiSelect');
+
+				if (ms !== undefined && ms.options.popup.parameters !== undefined) {
+					openSelectPopup($obj, event_target);
+
+					return false;
+				}
+			});
+		},
+
+		setCustomSuggestList: function(callback) {
+			this.each(function() {
+				const $obj = $(this);
+				const ms = $obj.data('multiSelect');
+
+				ms.options.custom_suggest_list = callback;
+			});
+		},
+
+		setSuggestListModifier: function(callback) {
+			this.each(function() {
+				const $obj = $(this);
+				const ms = $obj.data('multiSelect');
+
+				ms.options.suggest_list_modifier = callback;
+			});
+		},
+
+		customSuggestSelectHandler: function(callback) {
+			this.each(function() {
+				const $obj = $(this);
+				const ms = $obj.data('multiSelect');
+
+				ms.options.custom_suggest_select_handler = callback;
+			});
 		}
 	};
 
 	/**
-	 * Create multi select input element.
+	 * Initialize and interact with multi select input element.
 	 *
-	 * @param string options['url']					backend url
-	 * @param string options['name']				input element name
-	 * @param object options['labels']				translated labels (optional)
-	 * @param object options['data']				preload data {id, name, prefix} (optional)
-	 * @param string options['data'][id]
-	 * @param string options['data'][name]
-	 * @param string options['data'][prefix]		(optional)
-	 * @param bool   options['data'][inaccessible]	(optional)
-	 * @param bool   options['data'][disabled]		(optional)
-	 * @param string options['placeholder']			set custom placeholder (optional)
-	 * @param array  options['excludeids']			the list of excluded ids (optional)
-	 * @param string options['defaultValue']		default value for input element (optional)
-	 * @param bool   options['disabled']			turn on/off readonly state (optional)
-	 * @param bool   options['addNew']				allow user to create new names (optional)
-	 * @param int    options['selectedLimit']		how many items can be selected (optional)
-	 * @param int    options['limit']				how many available items can be received from backend (optional)
-	 * @param object options['popup']				popup data {parameters, width, height} (optional)
-	 * @param string options['popup']['parameters']
-	 * @param string options['popup']['filter_preselect_fields']
-	 * @param int    options['popup']['width']
-	 * @param int    options['popup']['height']
-	 * @param object options['autosuggest']         autosuggest options (optional)
-	 * @param object options['autosuggest']['filter_preselect_fields'] autosuggest preselect fields (optional)
-	 * @param string options['autosuggest']['filter_preselect_fields']['hosts'] autosuggest host preselect field (optional)
-	 * @param string options['styles']				additional style for multiselect wrapper HTML element (optional)
-	 * @param string options['styles']['property']
-	 * @param string options['styles']['value']
+	 * Function can either accept a method from supported methods or expects the multiselect element to contain
+	 * a 'data-params' attribute with the following possible properties:
+	 *     string url                   backend url
+	 *     string name                  input element name
+	 *     string multiselect_id        multiselect wrapper id (optional)
+	 *     object labels                translated labels (optional)
+	 *     array  data                  preload data {id, name, prefix} (optional)
+	 *     string data[][id]
+	 *     string data[][name]
+	 *     string data[][prefix]        (optional)
+	 *     bool   data[][inaccessible]  (optional)
+	 *     bool   data[][disabled]      (optional)
+	 *     string placeholder           set custom placeholder (optional)
+	 *     string defaultValue          default value for input element (optional)
+	 *     bool   disabled              turn on/off disabled state (optional)
+	 *     bool   readonly              turn on/off readonly state (optional)
+	 *     bool   hidden                hide element (optional)
+	 *     bool   addNew                allow user to create new names (optional)
+	 *     int    maxlength             maximum length for new names (optional)
+	 *     int    selectedLimit         how many items can be selected (optional)
+	 *     int    limit                 how many available items can be received from backend (optional)
+	 *     object popup                 popup data {parameters, width, height} (optional)
+	 *     string popup[parameters]
+	 *     string popup[filter_preselect]
+	 *     string popup[filter_preselect][id]
+	 *     string popup[filter_preselect][submit_as]
+	 *     object popup[filter_preselect][submit_parameters]
+	 *     bool   popup[filter_preselect][multiple]
+	 *     int    popup[width]
+	 *     int    popup[height]
+	 *     object autosuggest           autosuggest options (optional)
+	 *     object autosuggest[filter_preselect]
+	 *     string autosuggest[filter_preselect][id]
+	 *     string autosuggest[filter_preselect][submit_as]
+	 *     object autosuggest[filter_preselect][submit_parameters]
+	 *     bool   autosuggest[filter_preselect][multiple]
+	 *     string styles                additional style for multiselect wrapper HTML element (optional)
+	 *     string styles[property]
+	 *     string styles[value]
 	 *
 	 * @return object
 	 */
-	$.fn.multiSelect = function(options) {
-		// Call a public method.
-		if (methods[options]) {
-			return methods[options].apply(this, Array.prototype.slice.call(arguments, 1));
+	$.fn.multiSelect = function(method) {
+		if (method !== undefined) {
+			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
 		}
 
 		var defaults = {
-				url: '',
-				name: '',
-				labels: {
-					'No matches found': t('No matches found'),
-					'More matches found...': t('More matches found...'),
-					'type here to search': t('type here to search'),
-					'new': t('new'),
-					'Select': t('Select')
-				},
-				placeholder: t('type here to search'),
-				data: [],
-				only_hostid: 0,
-				excludeids: [],
-				addNew: false,
-				defaultValue: null,
-				disabled: false,
-				selectedLimit: 0,
-				limit: 20,
-				popup: {},
-				styles: {}
-			};
-
-		options = $.extend({}, defaults, options);
+			url: '',
+			name: '',
+			multiselect_id: '',
+			object_labels: {object: '', objects: ''},
+			labels: {
+				'No matches found': t('No matches found'),
+				'More matches found...': t('More matches found...'),
+				'type here to search': t('type here to search'),
+				'new': t('new'),
+				'Select': t('Select')
+			},
+			placeholder: t('type here to search'),
+			data: [],
+			addNew: false,
+			maxlength: null,
+			defaultValue: null,
+			custom_select: false,
+			custom_suggest_list: null,
+			suggest_list_modifier: null,
+			custom_suggest_select_handler: null,
+			disabled: false,
+			readonly: false,
+			selectedLimit: 0,
+			limit: 20,
+			popup: {},
+			styles: {}
+		};
 
 		return this.each(function() {
 			var $obj = $(this);
@@ -319,26 +462,31 @@ jQuery(function($) {
 				return;
 			}
 
+			const options = $.extend({}, defaults, JSON.parse(this.dataset.params));
+
 			options.required_str = $obj.attr('aria-required') === undefined ? 'false' : $obj.attr('aria-required');
 			$obj.removeAttr('aria-required');
+			$obj.attr('data-field-type', 'multiselect');
 
 			var ms = {
-					options: options,
-					values: {
-						search: '',
-						searches: {},
-						searching: {},
-						selected: {},
-						available: {},
-						available_div: $('<div>', {'class': 'multiselect-available'}),
+				options: options,
+				values: {
+					search: '',
+					searches: {},
+					searching: {},
+					selected: {},
+					available: new Map(),
+					available_div: $('<div>', {'class': 'multiselect-available'}),
 
-						/*
-						 * Indicates a false click on an available list, but not on some actual item.
-						 * In such case the "focusout" event (IE) of the search input should not be processed.
-						 */
-						available_false_click: false
-					}
-				};
+					/*
+					 * Indicates a false click on an available list, but not on some actual item.
+					 * In such case the "focusout" event (IE) of the search input should not be processed.
+					 */
+					available_false_click: false
+				},
+				select_button: null,
+				optional_select_menu: []
+			};
 
 			ms.values.available_div.on('mousedown', 'li', function() {
 				/*
@@ -352,23 +500,30 @@ jQuery(function($) {
 
 			$obj.wrap($('<div>', {
 				'class': ZBX_STYLE_CLASS,
-				css: ms.options.styles
+				css: ms.options.styles,
+				id: ms.options.multiselect_id !== '' ? ms.options.multiselect_id : null
 			}));
 
 			var $selected_div = $('<div>', {'class': 'selected'}).on('click', function() {
-					/*
-					 * Focus without options because here it don't matter.
-					 * Click used instead focus because in patternselect listen only click.
-					 */
-					$('input[type="text"]', $obj).click().focus();
-				}),
-				$selected_ul = $('<ul>', {'class': 'multiselect-list'});
+				/*
+				 * Focus without options because here it don't matter.
+				 * Click used instead focus because in patternselect listen only click.
+				 */
+				$('input[type="text"]', $obj).click().focus();
+			}),
+			$selected_ul = $('<ul>', {'class': 'multiselect-list'});
 
 			$obj.append($selected_div.append($selected_ul));
 
-			if (ms.options.disabled) {
-				$obj.attr('aria-disabled', true);
-				$selected_ul.addClass('disabled');
+			if (ms.options.disabled || ms.options.readonly) {
+				if (ms.options.disabled) {
+					$obj.attr('aria-disabled', true);
+					$selected_ul.addClass('disabled');
+				}
+				else {
+					$obj.attr('aria-readonly', true);
+					$selected_ul.attr('aria-readonly', true);
+				}
 			}
 			else {
 				$obj.append(makeMultiSelectInput($obj));
@@ -376,7 +531,7 @@ jQuery(function($) {
 
 			$obj
 				.on('mousedown', function(event) {
-					if (isSearchFieldVisible($obj) && ms.options.selectedLimit != 1) {
+					if (isSearchFieldVisible($obj) && ms.options.selectedLimit != 1 && !ms.options.readonly) {
 						$obj.addClass('active');
 						$('.selected li.selected', $obj).removeClass('selected');
 
@@ -399,38 +554,24 @@ jQuery(function($) {
 				});
 			}
 
-			if (ms.options.popup.parameters != null) {
-				if (typeof ms.options.popup.parameters['only_hostid'] !== 'undefined') {
-					ms.options.only_hostid = ms.options.popup.parameters['only_hostid'];
-				}
-
-				var popup_button = $('<button>', {
-						type: 'button',
-						'class': 'btn-grey',
-						text: ms.options.labels['Select']
-					});
-
-				if (ms.options.disabled) {
-					popup_button.prop('disabled', true);
-				}
-
-				popup_button.on('click', function(event) {
-					var popup_options = ms.options.popup.parameters;
-
-					if (ms.options.popup.filter_preselect_fields) {
-						popup_options = jQuery.extend(popup_options, getFilterPreselectField($obj, MS_ACTION_POPUP));
-					}
-
-					if (typeof popup_options['disable_selected'] !== 'undefined' && popup_options['disable_selected']) {
-						popup_options['disableids'] = Object.keys(ms.values.selected);
-					}
-
-					// Click used instead focus because in patternselect listen only click.
-					$('input[type="text"]', $obj).click();
-					return PopUp('popup.generic', popup_options, null, event.target);
+			if (ms.options.custom_select || ms.options.popup.parameters !== undefined) {
+				ms.select_button = $('<button>', {
+					type: 'button',
+					'class': `${ZBX_STYLE_BTN_GREY} multiselect-button`,
+					text: ms.options.labels['Select']
 				});
+			}
 
-				$obj.after($('<div>', {'class': 'multiselect-button'}).append(popup_button));
+			if (ms.select_button !== null) {
+				if (ms.options.popup.parameters !== undefined) {
+					ms.select_button.on('click', (e) => openSelectPopup($obj, e.target));
+				}
+
+				if (ms.options.disabled || ms.options.readonly) {
+					ms.select_button.prop('disabled', true);
+				}
+
+				addSelectButton($obj, ms.select_button);
 			}
 		});
 	};
@@ -443,27 +584,37 @@ jQuery(function($) {
 	 *
 	 * @return {object}
 	 */
-	function getFilterPreselectField($obj, action) {
-		var ms = $obj.data('multiSelect'),
-			preselect_options = ms.options[(action == MS_ACTION_AUTOSUGGEST) ? 'autosuggest' : 'popup'] || null,
-			ret = {};
+	function getFilterPreselect($obj, action) {
+		const ms = $obj.data('multiSelect');
+		const options_key = action == MS_ACTION_AUTOSUGGEST ? 'autosuggest' : 'popup';
 
-		if (!preselect_options) {
-			return ret;
+		if (!(options_key in ms.options) || !('filter_preselect' in ms.options[options_key])) {
+			return {};
 		}
 
-		if (typeof preselect_options.filter_preselect_fields.hosts !== 'undefined') {
-			var hosts = $('#' + preselect_options.filter_preselect_fields.hosts).multiSelect('getData');
-			if (hosts.length != 0) {
-				ret.hostid = hosts[0].id;
+		const filter_preselect = ms.options[options_key].filter_preselect;
+		const data = $('#' + filter_preselect.id).multiSelect('getData');
+
+		const accept = filter_preselect.accept ?? null;
+
+		const ret_data = [];
+
+		for (const item of data) {
+			if (accept === null || (accept === FILTER_PRESELECT_ACCEPT_ID && /^[0-9]+$/g.test(item.id))) {
+				ret_data.push(item.id);
 			}
 		}
 
-		if (typeof preselect_options.filter_preselect_fields.hostgroups !== 'undefined') {
-			var host_groups = $('#' + preselect_options.filter_preselect_fields.hostgroups).multiSelect('getData');
-			if (host_groups.length != 0) {
-				ret.groupid = host_groups[0].id;
-			}
+		if (ret_data.length === 0) {
+			return {};
+		}
+
+		let ret = {
+			[filter_preselect.submit_as]: filter_preselect.multiple ? ret_data : ret_data[0]
+		};
+
+		if ('submit_parameters' in filter_preselect) {
+			ret = {...ret, ...filter_preselect.submit_parameters};
 		}
 
 		return ret;
@@ -480,7 +631,8 @@ jQuery(function($) {
 				'autocomplete': 'off',
 				'placeholder': ms.options.placeholder,
 				'aria-label': ($label.length ? $label.text() + '. ' : '') + ms.options.placeholder,
-				'aria-required': ms.options.required_str
+				'aria-required': ms.options.required_str,
+				'maxlength': ms.options.maxlength
 			})
 				.on('keyup', function(e) {
 					switch (e.which) {
@@ -508,7 +660,7 @@ jQuery(function($) {
 					}
 
 					if (search !== '') {
-						var preselect_values = getFilterPreselectField($obj, MS_ACTION_AUTOSUGGEST),
+						var preselect_values = getFilterPreselect($obj, MS_ACTION_AUTOSUGGEST),
 							cache_key = search + JSON.stringify(preselect_values);
 
 						/*
@@ -518,7 +670,14 @@ jQuery(function($) {
 						 * 3. Schedule result set retrieval for the given term otherwise.
 						 */
 
-						if (cache_key in ms.values.searches) {
+						if (ms.options.custom_suggest_list !== null) {
+							ms.values.search = search;
+							ms.values.cache_key = cache_key;
+							ms.values.searches[cache_key] = ms.options.custom_suggest_list();
+							loadAvailable($obj);
+							showAvailable($obj);
+						}
+						else if (cache_key in ms.values.searches) {
 							ms.values.search = search;
 							ms.values.cache_key = cache_key;
 							loadAvailable($obj);
@@ -535,7 +694,9 @@ jQuery(function($) {
 									cache: false,
 									data: jQuery.extend({
 										search: search,
-										limit: getLimit($obj),
+										limit: ms.options.limit !== 0
+											? ms.options.limit + getSkipSearchIds($obj).length + 1
+											: undefined
 									}, preselect_values)
 								})
 									.then(function(response) {
@@ -564,7 +725,13 @@ jQuery(function($) {
 					switch (e.which) {
 						case KEY_TAB:
 						case KEY_ESCAPE:
-							hideAvailable($obj);
+							var $available = ms.values.available_div;
+
+							if ($available.parent().is(document.body)) {
+								hideAvailable($obj);
+								e.stopPropagation();
+							}
+
 							cleanSearchInput($obj);
 							break;
 
@@ -573,7 +740,7 @@ jQuery(function($) {
 								var $selected = $('li.suggest-hover', ms.values.available_div);
 
 								if ($selected.length) {
-									select($obj, $selected.data('id'));
+									select($obj, $selected[0].dataset.id);
 									$aria_live.text(sprintf(t('Added, %1$s'), $selected.data('label')));
 								}
 
@@ -615,13 +782,15 @@ jQuery(function($) {
 
 						case KEY_ARROW_UP:
 						case KEY_ARROW_DOWN:
-							var $collection = $('li', ms.values.available_div.filter(':visible')),
-								$selected = $collection.filter('.suggest-hover').removeClass('suggest-hover');
+							var $collection = $('li[data-id]', ms.values.available_div.filter(':visible')),
+								$selected = $collection.filter('.suggest-hover');
 
 							if ($selected.length) {
+								$selected.removeClass('suggest-hover');
+
 								$selected = (e.which == KEY_ARROW_UP)
-									? ($selected.is(':first-child') ? $collection.last() : $selected.prev())
-									: ($selected.is(':last-child') ? $collection.first() : $selected.next());
+									? ($selected.is($collection.first()) ? $collection.last() : $selected.prevAll('[data-id]').first())
+									: ($selected.is($collection.last()) ? $collection.first() : $selected.nextAll('[data-id]').first());
 
 								$selected.addClass('suggest-hover');
 								$aria_live.text($selected.data('label'));
@@ -637,22 +806,28 @@ jQuery(function($) {
 								var $selected = $('.selected li.selected', $obj);
 
 								if ($selected.length) {
-									var id = $selected.data('id'),
-										item = ms.values.selected[id];
+									const id = $selected[0].dataset.id;
+									const item = ms.values.selected[id];
 
-									if (typeof item.disabled === 'undefined' || !item.disabled) {
-										var aria_text = sprintf(t('Removed, %1$s'), $selected.data('label'));
+									if (item.disabled === undefined || !item.disabled) {
+										let aria_text = sprintf(t('Removed, %1$s'), $selected.data('label'));
 
 										$selected = (e.which == KEY_BACKSPACE)
-											? ($selected.is(':first-child') ? $selected.next() : $selected.prev())
-											: ($selected.is(':last-child') ? $selected.prev() : $selected.next());
+											? ($selected.is(':first-child')
+												? $selected.next('[data-id]')
+												: $selected.prev('[data-id]')
+											)
+											: ($selected.is(':last-child')
+												? $selected.prev('[data-id]')
+												: $selected.next('[data-id]')
+											);
 
 										removeSelected($obj, id);
 
 										$obj.trigger('change', ms);
 
 										if ($selected.length) {
-											var $collection = $('.selected li', $obj);
+											var $collection = $('.selected li[data-id]', $obj);
 											$selected.addClass('selected');
 
 											aria_text += ', ' + sprintf(
@@ -668,7 +843,7 @@ jQuery(function($) {
 										$aria_live.text(aria_text);
 									}
 									else {
-										$aria_live.text(t('Can not be removed'));
+										$aria_live.text(t('Cannot be removed'));
 									}
 								}
 								else if (e.which == KEY_BACKSPACE) {
@@ -685,10 +860,10 @@ jQuery(function($) {
 							break;
 					}
 				})
-				.on('focusin', function($event) {
+				.on('focusin', function() {
 					$obj.addClass('active');
 				})
-				.on('focusout', function($event) {
+				.on('focusout', function() {
 					if (ms.values.available_false_click) {
 						ms.values.available_false_click = false;
 						$('input[type="text"]', $obj)[0].focus({preventScroll:true});
@@ -724,7 +899,12 @@ jQuery(function($) {
 	function select($obj, id) {
 		var ms = $obj.data('multiSelect');
 
-		addSelected($obj, ms.values.available[id]);
+		if (ms.options.custom_suggest_select_handler !== null) {
+			ms.options.custom_suggest_select_handler(ms.values.available.get(id.toString()));
+		}
+		else {
+			addSelected($obj, ms.values.available.get(id.toString()));
+		}
 
 		if (isSearchFieldVisible($obj)) {
 			$('input[type="text"]', $obj)[0].focus({preventScroll:true});
@@ -733,6 +913,59 @@ jQuery(function($) {
 		$obj.trigger('change', ms);
 	}
 
+	function addSelectButton($obj, $button) {
+		let $container = $obj.siblings('.btn-split');
+
+		if (!$container.length) {
+			$container = $('<ul>', {'class': 'btn-split'});
+			$obj.after($container);
+		}
+
+		$container.append($('<li>').append($button));
+	}
+
+	function addOptionalSelect($obj, label, callback) {
+		const ms = $obj.data('multiSelect');
+
+		if (!ms.optional_select_menu.length) {
+			addSelectButton($obj, $('<button>', {
+				type: 'button',
+				class: `${ZBX_STYLE_BTN_GREY} ${ZBX_ICON_CHEVRON_DOWN_SMALL} multiselect-optional-select-button`
+			}).on('click', function(event) {
+				jQuery(event.target).menuPopup(
+					[{items: ms.optional_select_menu}],
+					new jQuery.Event(event), {
+						position: {of: event.target, my: 'left top', at: 'left bottom', within: '.wrapper'}
+					}
+				);
+			}));
+		}
+
+		ms.optional_select_menu.push({label, clickCallback: callback});
+	}
+
+	function openSelectPopup($obj, open_trigger_element) {
+		const ms = $obj.data('multiSelect');
+
+		let parameters = ms.options.popup.parameters;
+
+		if (ms.options.popup.filter_preselect) {
+			parameters = jQuery.extend(parameters, getFilterPreselect($obj, MS_ACTION_POPUP));
+		}
+
+		if (parameters['disable_selected'] !== undefined && parameters['disable_selected']) {
+			parameters['disableids'] = Object.keys(ms.values.selected);
+		}
+
+		// Click used instead focus because in pattern select only click is listened for.
+		$('input[type="text"]', $obj).click();
+
+		PopUp('popup.generic', parameters, {
+			dialogue_class: 'modal-popup-generic',
+			trigger_element: open_trigger_element
+		});
+	}
+
 	function addSelected($obj, item) {
 		var ms = $obj.data('multiSelect');
 
@@ -746,9 +979,11 @@ jQuery(function($) {
 		var prefix = (item.prefix || ''),
 			item_disabled = (typeof item.disabled !== 'undefined' && item.disabled);
 
+		const new_item_name = ms.options.newItemName ?? ms.options.name + '[new]';
+
 		$obj.append($('<input>', {
 			type: 'hidden',
-			name: (ms.options.addNew && item.isNew) ? ms.options.name + '[new]' : ms.options.name,
+			name: (ms.options.addNew && item.isNew) ? new_item_name : ms.options.name,
 			value: item.id,
 			'data-name': item.name,
 			'data-prefix': prefix
@@ -767,9 +1002,9 @@ jQuery(function($) {
 							title: item.name
 						}))
 						.append($('<span>')
-							.addClass('subfilter-disable-btn')
+							.addClass([ZBX_STYLE_BTN_ICON, ZBX_ICON_REMOVE_SMALLER])
 							.on('click', function() {
-								if (!ms.options.disabled && !item_disabled) {
+								if (!ms.options.disabled && !item_disabled && !ms.options.readonly) {
 									removeSelected($obj, item.id);
 									if (isSearchFieldVisible($obj)) {
 										$('input[type="text"]', $obj)[0].focus({preventScroll:true});
@@ -781,7 +1016,7 @@ jQuery(function($) {
 						)
 				)
 				.on('click', function() {
-					if (isSearchFieldVisible($obj) && ms.options.selectedLimit != 1) {
+					if (isSearchFieldVisible($obj) && ms.options.selectedLimit != 1 && !ms.options.readonly) {
 						$('.selected li.selected', $obj).removeClass('selected');
 						$(this).addClass('selected');
 
@@ -804,14 +1039,16 @@ jQuery(function($) {
 	}
 
 	function removeSelected($obj, id) {
-		var ms = $obj.data('multiSelect');
+		const ms = $obj.data('multiSelect');
 
-		$('.selected li[data-id]', $obj).each(function(){
-			if ($(this).data('id') == id) {
+		$obj.trigger('before-remove', ms);
+
+		$('.multiselect-list [data-id]', $obj).each(function() {
+			if (this.dataset.id == id) {
 				$(this).remove();
 			}
 		});
-		$('input', $obj).each(function(){
+		$('input[type="hidden"]', $obj).each(function() {
 			if ($(this).val() == id) {
 				$(this).remove();
 			}
@@ -819,7 +1056,7 @@ jQuery(function($) {
 
 		delete ms.values.selected[id];
 
-		if (!$('.selected li', $obj).length) {
+		if (!Object.keys(ms.values.selected).length) {
 			addDefaultValue($obj);
 		}
 
@@ -830,10 +1067,10 @@ jQuery(function($) {
 		var ms = $obj.data('multiSelect'),
 			is_new = item.isNew || false,
 			prefix = item.prefix || '',
-			search = ms.values.search.replace(/[.+?^${}()|[\]\\]/g, '\\$&').replace(/[*]/g, '\\\*?'),
 			$li = $('<li>', {
 				'data-id': item.id,
-				'data-label': prefix + item.name
+				'data-label': prefix + item.name,
+				'data-source': item.source
 			})
 				.on('mouseenter', function() {
 					$('li.suggest-hover', ms.values.available_div).removeClass('suggest-hover');
@@ -848,11 +1085,38 @@ jQuery(function($) {
 		}
 
 		// Highlight matched.
-		$li
-			.append(item.name.replace(new RegExp(search, 'gi'), function(match) {
-				return '<span' + (!is_new ? ' class="suggest-found"' : '') + '>' + match + '</span>';
-			}))
-			.toggleClass('suggest-new', is_new);
+		if (ms.values.search !== item.name) {
+			var text = item.name.toLowerCase(),
+				search = ms.values.search.toLowerCase().replace(/[*]+/g, ''),
+				start = 0,
+				end = 0;
+
+			while (search !== '' && text.indexOf(search, end) > -1) {
+				end = text.indexOf(search, end);
+
+				if (end > start) {
+					$li.append(document.createTextNode(item.name.substring(start, end)));
+				}
+
+				$li.append($('<span>', {
+					class: !is_new ? 'suggest-found' : '',
+					text: item.name.substring(end, end + search.length)
+				})).toggleClass('suggest-new', is_new);
+
+				end += search.length;
+				start = end;
+			}
+
+			if (end < item.name.length) {
+				$li.append(document.createTextNode(item.name.substring(end, item.name.length)));
+			}
+		}
+		else {
+			$li.append($('<span>', {
+				class: !is_new ? 'suggest-found' : '',
+				text: item.name
+			})).toggleClass('suggest-new', is_new);
+		}
 
 		$('ul', ms.values.available_div).append($li);
 	}
@@ -901,40 +1165,49 @@ jQuery(function($) {
 			}
 		}
 
-		var available_more = false;
+		const skip_search_ids = getSkipSearchIds($obj);
 
-		$.each(data, function(i, item) {
-			if (ms.options.limit == 0 || objectSize(ms.values.available) < ms.options.limit) {
-				if (typeof ms.values.available[item.id] === 'undefined'
-						&& typeof ms.values.selected[item.id] === 'undefined'
-						&& ms.options.excludeids.indexOf(item.id) === -1) {
-					ms.values.available[item.id] = item;
+		let available_more = false;
+
+		if (ms.options.custom_suggest_list !== null) {
+			ms.values.available = data;
+		}
+		else {
+			$.each(data, function(i, item) {
+				if (ms.options.limit == 0 || ms.values.available.size < ms.options.limit) {
+					if (!skip_search_ids.includes(item.id)) {
+						ms.values.available.set(item.id, item);
+					}
 				}
-			}
-			else {
-				available_more = true;
-			}
-		});
+				else {
+					available_more = true;
+				}
+			});
+		}
 
 		if (addNew) {
-			ms.values.available[ms.values.search] = {
+			ms.values.available.set(ms.values.search, {
 				id: ms.values.search,
 				name: ms.values.search + ' (' + ms.options.labels['new'] + ')',
 				isNew: true
-			};
+			});
 		}
 
 		var found = 0,
 			preselected = '';
 
-		if (objectSize(ms.values.available) == 0) {
+		if (ms.options.suggest_list_modifier !== null) {
+			ms.values.available = ms.options.suggest_list_modifier(ms.values.available);
+		}
+
+		if (ms.values.available.size === 0) {
 			var div = $('<div>', {
-					'class': 'multiselect-matches',
-					text: ms.options.labels['No matches found']
-				})
-					.on('click', function() {
-						$('input[type="text"]', $obj)[0].focus({preventScroll:true});
-					});
+				'class': 'multiselect-matches',
+				text: ms.options.labels['No matches found']
+			})
+				.on('click', function() {
+					$('input[type="text"]', $obj)[0].focus({preventScroll:true});
+				});
 
 			ms.values.available_div.append(div);
 		}
@@ -944,13 +1217,22 @@ jQuery(function($) {
 				'aria-hidden': true
 			}));
 
-			$.each(ms.values.available, function (i, item) {
-				if (found == 0) {
-					preselected = (item.prefix || '') + item.name;
+			for (const item of ms.values.available.values()) {
+				if ('group_label' in item) {
+					$('ul', ms.values.available_div)
+						.addClass('multiselect-suggest-grouped')
+						.append(
+							$('<li>', {class: 'suggest-group'}).text(item.group_label)
+						);
 				}
-				addAvailable($obj, item);
-				found++;
-			});
+				else {
+					if (found === 0) {
+						preselected = (item.prefix || '') + item.name;
+					}
+					addAvailable($obj, item);
+					found++;
+				}
+			}
 		}
 
 		if (found > 0) {
@@ -1016,7 +1298,7 @@ jQuery(function($) {
 				$(window).height() + $(window).scrollTop() - available_top - obj_padding_y - 10
 			));
 
-		if (objectSize(ms.values.available) > 0) {
+		if (ms.values.available.size > 0) {
 			available_width_min = Math.max(available_width, 300);
 
 			// Prevent less than 15% width difference for the available list and the input field.
@@ -1040,14 +1322,14 @@ jQuery(function($) {
 
 		$available.scrollTop(0);
 
-		if (objectSize(ms.values.available) != 0) {
+		if (ms.values.available.size !== 0) {
 			// Remove selected item selected state.
 			$('.selected li.selected', $obj).removeClass('selected');
 
 			// Pre-select first available item.
-			if ($('li', $available).length > 0) {
+			if ($('li[data-id]', $available).length > 0) {
 				$('li.suggest-hover', $available).removeClass('suggest-hover');
-				$('li:first-child', $available).addClass('suggest-hover');
+				$('li[data-id]', $available).first().addClass('suggest-hover');
 			}
 		}
 
@@ -1076,11 +1358,11 @@ jQuery(function($) {
 	}
 
 	function cleanAvailable($obj) {
-		var ms = $obj.data('multiSelect');
+		const ms = $obj.data('multiSelect');
 
 		hideAvailable($obj);
 
-		ms.values.available = {};
+		ms.values.available.clear();
 		ms.values.available_div.empty();
 	}
 
@@ -1094,7 +1376,7 @@ jQuery(function($) {
 				selected_top = 0,
 				selected_height = $selected.outerHeight(true);
 
-			if ($('.multiselect-matches', $available)) {
+			if ($('.multiselect-matches', $available).length > 0) {
 				selected_top += $('.multiselect-matches', $available).outerHeight(true);
 			}
 
@@ -1154,7 +1436,7 @@ jQuery(function($) {
 	function updateSearchFieldVisibility($obj) {
 		var ms = $obj.data('multiSelect'),
 			visible_now = !$obj.hasClass('search-disabled'),
-			visible = !ms.options.disabled
+			visible = !ms.options.disabled && !ms.options.readonly
 				&& (ms.options.selectedLimit == 0 || $('.selected li', $obj).length < ms.options.selectedLimit);
 
 		if (visible === visible_now) {
@@ -1187,11 +1469,13 @@ jQuery(function($) {
 		return !$obj.hasClass('search-disabled');
 	}
 
-	function getLimit($obj) {
-		var ms = $obj.data('multiSelect');
+	function getSkipSearchIds($obj) {
+		const ms = $obj.data('multiSelect');
 
-		return (ms.options.limit != 0)
-			? ms.options.limit + objectSize(ms.values.selected) + ms.options.excludeids.length + 1
-			: null;
+		return [...new Set([
+			...Object.keys(ms.values.selected),
+			...(ms.options.popup.parameters?.excludeids || []),
+			...(ms.options.popup.parameters?.disableids || [])
+		])];
 	}
-});
+})(jQuery);
